Chromium Code Reviews
chromiumcodereview-hr@appspot.gserviceaccount.com (chromiumcodereview-hr) | Please choose your nickname with Settings | Help | Chromium Project | Gerrit Changes | Sign out
(646)

Unified Diff: base/metrics/field_trial.cc

Issue 2530573002: Share field trial allocator on zygote-using Linuxes (Closed)
Patch Set: git rebase-update Created 4 years, 1 month ago
Use n/p to move between diff chunks; N/P to move between comments. Draft comments are only viewable by you.
Jump to:
View side-by-side diff with in-line comments
Download patch
Index: base/metrics/field_trial.cc
diff --git a/base/metrics/field_trial.cc b/base/metrics/field_trial.cc
index 1f18658abd72faf92b39ff982873a84c64ac7c91..f2e2e1fe189582e72795dcc3c034c51b49c55ad1 100644
--- a/base/metrics/field_trial.cc
+++ b/base/metrics/field_trial.cc
@@ -21,6 +21,10 @@
#include "base/strings/stringprintf.h"
#include "base/strings/utf_string_conversions.h"
+#if defined(POSIX_WITH_ZYGOTE)
+#include "base/posix/global_descriptors.h"
+#endif
+
namespace base {
namespace {
@@ -232,11 +236,20 @@ HANDLE CreateReadOnlyHandle(FieldTrialList::FieldTrialAllocator* allocator) {
DWORD access = SECTION_MAP_READ | SECTION_QUERY;
HANDLE dst;
if (!::DuplicateHandle(process, src, process, &dst, access, true, 0))
- return nullptr;
+ return kInvalidPlatformFile;
return dst;
}
#endif
+#if defined(POSIX_WITH_ZYGOTE)
+int CreateReadOnlyHandle(FieldTrialList::FieldTrialAllocator* allocator) {
+ SharedMemoryHandle new_handle;
+ allocator->shared_memory()->ShareReadOnlyToProcess(GetCurrentProcessHandle(),
+ &new_handle);
+ return SharedMemory::GetFdFromSharedMemoryHandle(new_handle);
+}
+#endif
+
} // namespace
// statics
@@ -736,25 +749,35 @@ bool FieldTrialList::CreateTrialsFromString(
// static
void FieldTrialList::CreateTrialsFromCommandLine(
const CommandLine& cmd_line,
- const char* field_trial_handle_switch) {
+ const char* field_trial_handle_switch,
+ const int field_trial_handle) {
global_->create_trials_from_command_line_called_ = true;
-#if defined(OS_WIN) && !defined(OS_NACL)
- if (cmd_line.HasSwitch(field_trial_handle_switch)) {
- std::string arg = cmd_line.GetSwitchValueASCII(field_trial_handle_switch);
- int field_trial_handle = std::stoi(arg);
- HANDLE handle = reinterpret_cast<HANDLE>(field_trial_handle);
- bool result = CreateTrialsFromWindowsHandle(handle);
- DCHECK(result);
- }
-#endif
-
if (cmd_line.HasSwitch(switches::kForceFieldTrials)) {
bool result = FieldTrialList::CreateTrialsFromString(
cmd_line.GetSwitchValueASCII(switches::kForceFieldTrials),
std::set<std::string>());
DCHECK(result);
+ return;
}
+
+#if defined(POSIX_WITH_ZYGOTE)
+ if (kUseSharedMemoryForFieldTrials) {
+ bool result = CreateTrialsFromFdKey(field_trial_handle);
+ DCHECK(result);
+ return;
+ }
+#endif
+
+#if defined(OS_WIN)
+ if (cmd_line.HasSwitch(field_trial_handle_switch)) {
+ std::string handle_switch =
+ cmd_line.GetSwitchValueASCII(field_trial_handle_switch);
+ bool result = CreateTrialsFromHandleSwitch(handle_switch);
+ DCHECK(result);
+ return;
+ }
+#endif
}
#if defined(OS_WIN)
@@ -771,6 +794,20 @@ void FieldTrialList::AppendFieldTrialHandleIfNeeded(
}
#endif
+#if defined(OS_POSIX) && !defined(OS_NACL)
+// static
+int FieldTrialList::GetFieldTrialHandle() {
+ if (!global_)
+ return kInvalidPlatformFile;
+ if (kUseSharedMemoryForFieldTrials) {
+ InstantiateFieldTrialAllocatorIfNeeded();
+ // We check for an invalid handle where this gets called.
+ return global_->readonly_allocator_handle_;
+ }
+ return kInvalidPlatformFile;
+}
+#endif
+
// static
void FieldTrialList::CopyFieldTrialStateToFlags(
const char* field_trial_handle_switch,
@@ -788,15 +825,16 @@ void FieldTrialList::CopyFieldTrialStateToFlags(
InstantiateFieldTrialAllocatorIfNeeded();
// If the readonly handle didn't get duplicated properly, then fallback to
// original behavior.
- if (!global_->readonly_allocator_handle_) {
+ if (global_->readonly_allocator_handle_ == kInvalidPlatformFile) {
AddForceFieldTrialsFlag(cmd_line);
return;
}
// HANDLE is just typedef'd to void *. We basically cast the handle into an
// int (uintptr_t, to be exact), stringify the int, and pass it as a
// command-line flag. The child process will do the reverse conversions to
// retrieve the handle. See http://stackoverflow.com/a/153077
auto uintptr_handle =
reinterpret_cast<uintptr_t>(global_->readonly_allocator_handle_);
std::string field_trial_handle = std::to_string(uintptr_handle);
@@ -894,9 +932,28 @@ size_t FieldTrialList::GetFieldTrialCount() {
#if defined(OS_WIN)
// static
-bool FieldTrialList::CreateTrialsFromWindowsHandle(HANDLE handle) {
+bool FieldTrialList::CreateTrialsFromHandleSwitch(
+ const std::string& handle_switch) {
+ int field_trial_handle = std::stoi(handle_switch);
+ HANDLE handle = reinterpret_cast<HANDLE>(field_trial_handle);
SharedMemoryHandle shm_handle(handle, GetCurrentProcId());
+ return FieldTrialList::CreateTrialsFromSharedMemoryHandle(shm_handle);
+}
+#endif
+
+#if defined(POSIX_WITH_ZYGOTE)
+// static
+bool FieldTrialList::CreateTrialsFromFdKey(const int fd_key) {
+ int fd = GlobalDescriptors::GetInstance()->Get(fd_key);
+ SharedMemoryHandle shm_handle(fd, true);
+ return FieldTrialList::CreateTrialsFromSharedMemoryHandle(shm_handle);
+}
+#endif
+#if !defined(OS_NACL)
+// static
+bool FieldTrialList::CreateTrialsFromSharedMemoryHandle(
+ SharedMemoryHandle shm_handle) {
// shm gets deleted when it gets out of scope, but that's OK because we need
// it only for the duration of this method.
std::unique_ptr<SharedMemory> shm(new SharedMemory(shm_handle, true));
@@ -986,8 +1043,15 @@ void FieldTrialList::InstantiateFieldTrialAllocatorIfNeeded() {
if (global_->field_trial_allocator_ != nullptr)
return;
+ SharedMemoryCreateOptions options;
+ options.size = kFieldTrialAllocationSize;
+ options.share_read_only = true;
+
std::unique_ptr<SharedMemory> shm(new SharedMemory());
- if (!shm->CreateAndMapAnonymous(kFieldTrialAllocationSize))
+ if (!shm->Create(options))
+ TerminateBecauseOutOfMemory(kFieldTrialAllocationSize);
+
+ if (!shm->Map(kFieldTrialAllocationSize))
TerminateBecauseOutOfMemory(kFieldTrialAllocationSize);
global_->field_trial_allocator_.reset(
@@ -999,7 +1063,7 @@ void FieldTrialList::InstantiateFieldTrialAllocatorIfNeeded() {
AddToAllocatorWhileLocked(registered.second);
}
-#if defined(OS_WIN)
+#if defined(OS_WIN) || defined(POSIX_WITH_ZYGOTE)
// Set |readonly_allocator_handle_| so we can pass it to be inherited and
// via the command line.
global_->readonly_allocator_handle_ =
@@ -788,15 +825,16 @@ void FieldTrialList::CopyFieldTrialStateToFlags(
return;
}
- // HANDLE is just typedef'd to void *. We basically cast the handle into an
- // int (uintptr_t, to be exact), stringify the int, and pass it as a
- // command-line flag. The child process will do the reverse conversions to
- // retrieve the handle. See http://stackoverflow.com/a/153077
+ // PlatformFile is typedef'd to HANDLE which is typedef'd to void *. We
+ // basically cast the handle into an int (uintptr_t, to be exact), stringify
+ // the int, and pass it as a command-line flag. The child process will do
+ // the reverse conversions to retrieve the handle. See
+ // http://stackoverflow.com/a/153077
auto uintptr_handle =
reinterpret_cast<uintptr_t>(global_->readonly_allocator_handle_);

Powered by Google App Engine
This is Rietveld 408576698